// UDA'Software 2003
// File: autoaway.cpp

/*

  Copyright (C) 1988 Free Software Foundation, Inc.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Questo script in C++ serve per convertire in stato di Away l'utente
 * che non utilizza le sessioni appartenenti allo stesso server per
 * piu' di un certo periodo.
 *
 * Per default l'AutoAway e' impostato a 900secondi (15min).
 *
 * La sintassi del comando e':
 *
 * /autoaway <idle_seconds> [command]
 *
 * dove:
 *
 * "idle_second" rappresenta il numero di secondi di inattivita' che
 * devono trascorrere prima che venga eseguito il comando di
 * AutoAway. Il valore 0 disabilita l'AutoAway.
 *
 * "command" rappresenta il comando che deve essere eseguito
 * dall'AutoAway. Per default e' "/AWAY AutoAway: I'm busy"
 *
 */

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Include la libreria C++ per la gestione degli X-Chat Plug-in.
 *
 */
#include "plugin.h"

#include <vector>
#include <time.h>

using namespace std;

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
#define AUTOAWAY_NO	0
#define AUTOAWAY_YES	1

/*
 * Dichiara a livello globale l'Oggetto Plugin.
 *
 * L'oggetto ingloba tutte le Proprieta' e le Azioni necessarie per la
 * gestione di un plug-in.
 *
 */
static Plugin	PluginObj;

/*
 * Dichiara e inizializza una struttura dati che ha il compito di
 * monitorare l'idle dell'utente locale su ogni singolo server.
 *
 */
struct ServerIdle {
  char		*psServer;	// Nome del Server
  time_t	tLastMessage;	// Tempo dell'ultima azione eseguita
  int		bAutoAway;	// Flag: Stabilisce se deve fare il
				// back dall'AutoAway.
};

typedef vector<struct ServerIdle*>	vServerIdle;
vServerIdle	vsiData;

/*
 * Dichiara ed inizializza 
 *
 */
int	iSecondIdle = 900;

/*
 * La variabile che contiene il comando da eseguire quando avviene
 * l'AutoAway.
 *
 */
char	*psAutoAwayCommand = strdup("AWAY AutoAway: I'm busy");

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce una funzione che viene invocata dalle varie callback al
 * fine di inserire il server che ha generato l'evento nell'apposito
 * array, settare l'idle e, se necessario, eseguire l'AutoAway.
 *
 */
void AutoAwayServerIdle(int AutoAway = 0) {
  struct ServerIdle	*psiData = 0;
  time_t		tCurrentTime = time(NULL);
  
  // Se il tempo di idle concesso e' diverso da zero provvede al
  // controllo, altrimenti salta completamente.
  if (iSecondIdle) {
    /*
     * Come prima cosa, se il vettore vsiData non risulta vuoto,
     * provvede a cercare il server; se lo trova, verifica che il tempo
     * di idle non sia superiore a quello previsto (se lo e', lancia il
     * comando AutoAway). Se non lo trova, provvede ad inserire un nuovo
     * record nel vettore.
     *
     */
    if (vsiData.size()) {
      for (unsigned int i = 0; i < vsiData.size(); i++) {
	if (!strcmp(vsiData[i]->psServer, 
		    PluginObj.get_info_server())) {
	  // Registra il server intercettato, in modo tale che non venga
	  // inserito ulteriormente.
	  psiData = vsiData[i];
	  
	  if (AutoAway) {
	    // Verifica se il tempo di idle supera il limite.
	    if (tCurrentTime - psiData->tLastMessage > iSecondIdle && 
		!PluginObj.get_info_away()) {
	      PluginObj.printf("---\002\0033 AutoAway on server\0038 %s\0033 (idle time:\0039 %d\0033sec)", 
			       PluginObj.get_info_server(), 
			       tCurrentTime - psiData->tLastMessage);

	      // Configura il flag; il plug-in esegue il back solo
	      // degli AWAY di cui e' responsabile.
	      psiData->bAutoAway = AUTOAWAY_YES;
	      i = vsiData.size();
	      
	      // Esegue il comando configurato.
	      PluginObj.command(psAutoAwayCommand);
	    }
	  } else {    
	    vsiData[i]->tLastMessage = tCurrentTime;
	  }
	}
      }
      
      // Se non ha trovato il server, provvede ad inserirlo.
      if (!psiData)
	goto insert_Data;
      
    } else {
    insert_Data:
      psiData = new ServerIdle[sizeof (struct ServerIdle)];
      memset(psiData, 0, sizeof(struct ServerIdle));
      
      psiData->psServer = strdup(PluginObj.get_info_server());
      psiData->tLastMessage = time(NULL);
      psiData->bAutoAway = AUTOAWAY_NO;
      vsiData.push_back(psiData);
    }
  }
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce la funzione cb_AutoAway(), che viene invocata ogni
 * volta che l'utente lancia il comando: /autoaway
 *
 */  
static int cb_AutoAway(char *word[], char *word_eol[], void *userdata) {
  // Verifica che i parametri passati siano corretti; se lo sono,
  // imposta quelli del plug-in.
  if (word[2][0] != 0) {
    iSecondIdle = atoi(word[2]);

    if (!iSecondIdle || iSecondIdle >= 60) {
      if (word[3][0] != 0)
	psAutoAwayCommand = strdup(word_eol[3]);
      
      PluginObj.printf("\002\0033AUTOAWAY: \0035idle_seconds =\0038 %d\0033 | \0036command =\0038 %s", 
		       iSecondIdle, 
		       psAutoAwayCommand);
    } else
      PluginObj.print(" \002\0034Sorry, but the idle time is too small (min 60sec or 0 to disable).");
  } else {
    PluginObj.print(" \002\0034Sorry, You haven't specified sufficient parameters.");
    PluginObj.printf("\002\0033AUTOAWAY: \0035idle_seconds =\0038 %d\0033 | \0036command =\0038 %s", 
		     iSecondIdle, 
		     psAutoAwayCommand);
    PluginObj.print("Usage: AUTOAWAY <idle_seconds> [command]");
  }

  return XCHAT_EAT_ALL;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * cb_RAWLINE() e' una callback speciale perche' viene invocata per
 * ogni messaggio che il server IRC spedisce al client.
 *
 */
static int cb_RAWLINE(char *word[], char *word_eol[], void *userdata) {
  AutoAwayServerIdle(AUTOAWAY_YES);
  
  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * DA QUI IN POI SONO DEFINITE LE CALLBACK INVOCATE IN BASE AGLI
 * EVENTI "NORMALI".
 *
 */
// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelAction(char *word[], void *userdata) {
  // *$1 $2
  // $1 Nickname
  // $2 The action

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelActionHilight(char *word[], void *userdata) {
  // *$1 $2
  // $1 Nickname
  // $2 The action

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelBan(char *word[], void *userdata) {
  // $1 sets ban on $2
  // $1 The nick of the person who did the banning
  // $2 The ban mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelDeHalfOp(char *word[], void *userdata) {
  // $1 removes channel half-operator status from $2
  // $1 The nick of the person of did the dehalfop'ing
  // $2 The nick of the person who has been dehalfop'ed

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelDeOp(char *word[], void *userdata) {
  // $1 removes channel operator status from $2
  // $1 The nick of the person of did the deop'ing
  // $2 The nick of the person who has been deop'ed

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelDeVoice(char *word[], void *userdata) {
  // $1 removes voice from $2
  // $1 The nick of the person of did the devoice'ing
  // $2 The nick of the person who has been devoice'ed

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelExempt(char *word[], void *userdata) {
  // $1 sets exempt on $2
  // $1 The nick of the person who did the exempt
  // $2 The exempt mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelHalf_Operator(char *word[], void *userdata) {
  // $1 gives channel half-operator status to $2
  // $1 The nick of the person who has been halfop'ed
  // $2 The nick of the person who did the halfop'ing

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelINVITE(char *word[], void *userdata) {
  // $1 sets invite on $2
  // $1 The nick of the person who did the invite
  // $2 The invite mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelMessage(char *word[], void *userdata) {
  // <$1>$2
  // $1 Nickname
  // $2 The text
  // $3 Mode char

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelModeGeneric(char *word[], void *userdata) {
  // $1 sets mode $2$3 $4
  // $1 The nick of the person setting the mode
  // $2 The mode's sign (+/-)
  // $3 The mode letter
  // $4 The channel it's being set on

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelMsgHilight(char *word[], void *userdata) {
  // <$1>$2

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelNotice(char *word[], void *userdata) {
  // -$1/$2-$3
  // $1 Who it's from
  // $2 The Channel it's going to
  // $3 The message

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelOperator(char *word[], void *userdata) {
  // $1 gives channel operator status to $2
  // $1 The nick of the person who did the op'ing
  // $2 The nick of the person who has been op'ed

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelRemoveExempt(char *word[], void *userdata) {
  // $1 removes exempt on $2
  // $1 The nick of the person removed the exempt
  // $2 The exempt mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelRemoveInvite(char *word[], void *userdata) {
  // $1 removes invite on $2
  // $1 The nick of the person removed the invite
  // $2 The invite mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelRemoveKeyword(char *word[], void *userdata) {
  // $1 removes user limit
  // $1 The nick who removed the key

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelRemoveLimit(char *word[], void *userdata) {
  // $1 sets channel keyword to $2
  // $1 The nick who removed the limit

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelSetKey(char *word[], void *userdata) {
  // $1 sets channel limit to $2
  // $1 The nick of the person who set the key
  // $2 The key

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelSetLimit(char *word[], void *userdata) {
  // $1 sets channel limit to $2
  // $1 The nick of the person who set the limit
  // $2 The limit

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelUnBan(char *word[], void *userdata) {
  // $1 removes ban on $2
  // $1 The nick of the person of did the unban'ing
  // $2 The ban mask

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ChannelVoice(char *word[], void *userdata) {
  // $1 gives voice to $2
  // $1 The nick of the person who did the voice'ing
  // $2 The nick of the person who has been voice'ed

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_CTCPSend(char *word[], void *userdata) {
  // >$1<CTCP $2
  // $1 Receiver
  // $2 Message

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_Kick(char *word[], void *userdata) {
  // <--$1 has kicked $2 from $3 ($4)
  // $1 The nickname of the kicker
  // $2 The person being kicked
  // $3 The channel
  // $4 The reason
  
  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_MessageSend(char *word[], void *userdata) {
  // >$1<$2

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_NoticeSend(char *word[], void *userdata) {
  // >$1<$2

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_PrivateMessage(char *word[], void *userdata) {
  // *$1*$2
  // $1 Nickname
  // $2 The message

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_PrivateMessagetoDialog(char *word[], void *userdata) {
  // <$1>$2
  // $1 Nickname
  // $2 The message

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_RawModes(char *word[], void *userdata) {
  // $1 sets modes [$2]
  // $1 Nickname
  // $2 Modes string

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_ServerConnected(char *word[], void *userdata) {
  // Connected.

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_TopicCreation(char *word[], void *userdata) {
  // Topic for $1 set by $2 at $3
  // $1 The channel
  // $2 The creator
  // $3 The time

  if (!PluginObj.nickcmp(word[2], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_TopicChange(char *word[], void *userdata) {
  // $1 has changed the topic to: $2
  // $1 Nick of person who changed the topic
  // $2 Topic
  // $3 Channel

  if (!PluginObj.nickcmp(word[1], PluginObj.get_info_nick()))
    AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YouJoin(char *word[], void *userdata) {
  // -->You are now talking on $2
  // $1 The nick of the joining person
  // $2 The channel being joined
  // $3 The host of the person

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YouPart(char *word[], void *userdata) {
  // You have left channel $3
  // $1 The nick of the person leaving
  // $2 The host of the person
  // $3 The channel

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YouPartwithReason(char *word[], void *userdata) {
  // You have left channel $3 ($4)
  // $1 The nick of the person leaving
  // $2 The host of the person
  // $3 The channel
  // $4 The reason

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YourMessage(char *word[], void *userdata) {
  // <$1>$2
  // $1 Nickname
  // $2 The text
  // $3 Mode char

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
static int cb_YourNickChanging(char *word[], void *userdata) {
  // You are now known as $2
  // $1 Old nickname
  // $2 New nickname

  AutoAwayServerIdle();

  return XCHAT_EAT_NONE;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce la funzione di inizializzazione xchat_plugin_init().
 *
 * Si tratta della prima funzione che viene automaticamente invocata
 * all'atto del caricamento del plug-in da parte del client X-Chat.
 *
 * I Parametri passati sono rispettivamente:
 *
 * - Un puntatore al plugin in memoria (necessario per la gestione del
     plug-in stesso).
 *
 * - Un riferimento a puntatore che dovra' contenere il Nome del
     plugin (da configurare nella funzione stessa).
 *
 * - Un riferimento a puntatore che dovra' contenere la Descrizione
     del plugin (da configurare nella funzione stessa).
 *
 * - Un riferimento a puntatore che dovra' contenere la Versione del
     plugin (da configurare nella funzione stessa).
 *
 * - Gli argomenti passati ad X-Chat dalla linea di comando.
 *
 */
extern "C" {
  int xchat_plugin_init(xchat_plugin *plugin_handle,
			char **plugin_name,
			char **plugin_desc,
			char **plugin_version,
			char *arg) {
    // Configura i parametri base del plugin.
    *plugin_name = strdup("AutoAway");
    *plugin_desc = strdup("Turn in Away mode automatically");
    *plugin_version = strdup("1.0.00");
    
    // Inizializza l'Oggetto plugin con i parametri base configurati.
    PluginObj = Plugin(plugin_handle, 
		       (*plugin_name), 
		       (*plugin_desc), 
		       (*plugin_version), 
		       arg);

    /*
     * Come prima cosa verifica se la versione di X-Chat e'
     * supportata; infatti, solo a partire dalla versione 2.0.2, sono
     * supportati i "Tipi di sessione" quindi la 2.0.0 e la 2.0.1
     * devono essere respinte.
     *
     */
    if ((strstr(PluginObj.get_info_version(), "2.0.0") || 
	 strstr(PluginObj.get_info_version(), "2.0.1")) && 
	strstr(PluginObj.get_info_version(), "2.0.") + 2 == '\0') {
      PluginObj.
	printf("\002\0034Sorry, but this version (\0033 %s \0034) isn't supported by this plug-in !", 
	       PluginObj.get_info_version());
      
      return 0;	/* ritorna 0 per fallimento */
    }

    // Aggancia l'evento "RAW LINE". Ogni messaggio proveniente dal
    // server IRC verra spedito alla callback cb_RAWLINE().
    PluginObj.hook_server("PONG", cb_RAWLINE);

    /*
     * Qui vengono agganciati tutti glie eventi gestiti dal client,
     * che sono anche contenuti nella lista ottenibile dalla finestra:
     * Settings->Lists->Text Events...
     *
     */    
    PluginObj.hook_print("Channel Action",			cb_ChannelAction);
    PluginObj.hook_print("Channel Action Hilight",		cb_ChannelActionHilight);
    PluginObj.hook_print("Channel Ban",				cb_ChannelBan);
    PluginObj.hook_print("Channel DeHalfOp",			cb_ChannelDeHalfOp);
    PluginObj.hook_print("Channel DeOp",			cb_ChannelDeOp);
    PluginObj.hook_print("Channel DeVoice",			cb_ChannelDeVoice);
    PluginObj.hook_print("Channel Exempt",			cb_ChannelExempt);
    PluginObj.hook_print("Channel Half-Operator",		cb_ChannelHalf_Operator);
    PluginObj.hook_print("Channel INVITE",			cb_ChannelINVITE);
    PluginObj.hook_print("Channel Message",			cb_ChannelMessage);
    PluginObj.hook_print("Channel Mode Generic",		cb_ChannelModeGeneric);
    PluginObj.hook_print("Channel Msg Hilight",			cb_ChannelMsgHilight);
    PluginObj.hook_print("Channel Notice",			cb_ChannelNotice);
    PluginObj.hook_print("Channel Operator",			cb_ChannelOperator);
    PluginObj.hook_print("Channel Remove Exempt",		cb_ChannelRemoveExempt);
    PluginObj.hook_print("Channel Remove Invite",		cb_ChannelRemoveInvite);
    PluginObj.hook_print("Channel Remove Keyword",		cb_ChannelRemoveKeyword);
    PluginObj.hook_print("Channel Remove Limit",		cb_ChannelRemoveLimit);
    PluginObj.hook_print("Channel Set Key",			cb_ChannelSetKey);
    PluginObj.hook_print("Channel Set Limit",			cb_ChannelSetLimit);
    PluginObj.hook_print("Channel UnBan",			cb_ChannelUnBan);
    PluginObj.hook_print("Channel Voice",			cb_ChannelVoice);
    PluginObj.hook_print("CTCP Send",				cb_CTCPSend);
    PluginObj.hook_print("Kick",				cb_Kick);
    PluginObj.hook_print("Message Send",			cb_MessageSend);
    PluginObj.hook_print("Notice Send",				cb_NoticeSend);
    PluginObj.hook_print("Private Message",			cb_PrivateMessage);
    PluginObj.hook_print("Private Message to Dialog",		cb_PrivateMessagetoDialog);
    PluginObj.hook_print("Raw Modes",				cb_RawModes);
    PluginObj.hook_print("Server Connected",			cb_ServerConnected);
    PluginObj.hook_print("Topic Creation",			cb_TopicCreation);
    PluginObj.hook_print("Topic Change",			cb_TopicChange);
    PluginObj.hook_print("You Join",				cb_YouJoin);
    PluginObj.hook_print("You Part",				cb_YouPart);
    PluginObj.hook_print("You Part with Reason",		cb_YouPartwithReason);
    PluginObj.hook_print("Your Message",			cb_YourMessage);
    PluginObj.hook_print("Your Nick Changing",			cb_YourNickChanging);
    
    /*
     * Aggancia i comandi del plug-in
     *
     */    
    PluginObj.hook_command("AUTOAWAY", 
			   cb_AutoAway,
			   "Usage: AUTOAWAY <idle_seconds> [command]");
    
    /*
     * Comunica il caricamento del plug-in e quali comandi sono stati
     * aggiunti.
     *
     */    
    PluginObj.print("\002\0033Autoaway\0036 Plugin loaded successfully !\n");
    PluginObj.printf("%s", "Command %BAUTOAWAY%O added.\n");

    return 1;	/* ritorna 1 per successo */
  }

  /*
   * Definisce la funzione di de-inizializzazione del plug-in. Viene
   * invocata ogni volta che il modulo viene scaricato dalla memoria.
   *
   */
  int xchat_plugin_deinit(xchat_plugin *plugin_handle) {
    PluginObj.print("\002\0033AutoAway\0036 Plugin unloaded.\n");

    return 1;       /* ritorna 1 per successo */
  }
}
